@@ -7,16 +7,166 @@ import gzip
 import pandas as pd
 import numpy as np
 import subprocess
+import time
+import argparse
+from datetime import timedelta
 from collections import defaultdict
+from functools import wraps
+from collections import Counter
 import celescope.tools
 import matplotlib
 matplotlib.use('Agg')
 import matplotlib.pyplot as plt
+from celescope.tools.__init__ import __PATTERN_DICT__
 
 tools_dir = os.path.dirname(celescope.tools.__file__)
-logger1 = logging.getLogger(__name__)
 
 
+def log(func):
+    '''
+    return logger.
+    logging start and done.
+    '''
+    logging.basicConfig(
+        level=logging.INFO,
+        format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
+    )
+    module = func.__module__
+    name = func.__name__
+    logger_name = f'{module}.{name}'
+    logger = logging.getLogger(logger_name)
+
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        logger.info('start...')
+        start = time.time()
+        result = func(*args, **kwargs)
+        end = time.time()
+        used = timedelta(seconds=end - start)
+        logger.info(f'done. time used: {used}')
+        return result
+
+    wrapper.logger = logger
+    return wrapper
+
+
+def read_barcode_file(match_dir):
+    '''
+    multi version compatible
+    '''
+    match_barcode_file1 = glob.glob(
+        f"{match_dir}/05.count/*_cellbarcode.tsv")
+    match_barcode_file2 = glob.glob(
+        f"{match_dir}/05.count/*matrix_10X/*_cellbarcode.tsv")
+    match_barcode_file3 = glob.glob(
+        f"{match_dir}/05.count/*matrix_10X/*barcodes.tsv")
+    match_barcode_file = (
+        match_barcode_file1 +
+        match_barcode_file2 +
+        match_barcode_file3)[0]
+    match_barcode, cell_total = read_one_col(match_barcode_file)
+    match_barcode = set(match_barcode)
+    return match_barcode, cell_total
+
+
+def format_stat(count, total_count):
+    percent = round(count / total_count * 100, 2)
+    string = f'{count}({percent}%)'
+    return string
+
+
+def multi_opts(assay):
+    readme = f'{assay} multi-samples'
+    parser = argparse.ArgumentParser(readme)
+    parser.add_argument('--mod', help='mod, sjm or shell', choices=['sjm', 'shell'], default='sjm')
+    parser.add_argument(
+        '--mapfile',
+        help='''
+            tsv file, 4 columns:
+            1st col: LibName;
+            2nd col: DataDir;
+            3rd col: SampleName;
+            4th col: Cell number or match_dir, optional;
+        ''',
+        required=True)
+    parser.add_argument('--chemistry', choices=__PATTERN_DICT__.keys(), help='chemistry version')
+    parser.add_argument('--whitelist', help='cellbarcode list')
+    parser.add_argument('--linker', help='linker')
+    parser.add_argument('--pattern', help='read1 pattern')
+    parser.add_argument('--outdir', help='output dir', default="./")
+    parser.add_argument(
+        '--adapt',
+        action='append',
+        help='adapter sequence',
+        default=[
+            'polyT=A{15}',
+            'p5=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC'])
+    parser.add_argument(
+        '--minimum-length',
+        dest='minimum_length',
+        help='minimum_length',
+        default=20)
+    parser.add_argument(
+        '--nextseq-trim',
+        dest='nextseq_trim',
+        help='nextseq_trim',
+        default=20)
+    parser.add_argument(
+        '--overlap',
+        help='minimum overlap length, default=5',
+        default=5)
+    parser.add_argument(
+        '--lowQual',
+        type=int,
+        help='max phred of base as lowQual',
+        default=0)
+    parser.add_argument(
+        '--lowNum',
+        type=int,
+        help='max number with lowQual allowed',
+        default=2)
+    parser.add_argument(
+        '--rm_files',
+        action='store_true',
+        help='remove redundant fq.gz and bam after running')
+    return parser
+
+
+def link_data(outdir, fq_dict):
+    raw_dir = f'{outdir}/data_give/rawdata'
+    os.system('mkdir -p %s' % (raw_dir))
+    with open(raw_dir + '/ln.sh', 'w') as fh:
+        fh.write('cd %s\n' % (raw_dir))
+        for s, arr in fq_dict.items():
+            fh.write('ln -sf %s %s\n' % (arr[0], s + '_1.fq.gz'))
+            fh.write('ln -sf %s %s\n' % (arr[1], s + '_2.fq.gz'))
+
+
+def gene_convert(gtf_file):
+
+    gene_id_pattern = re.compile(r'gene_id "(\S+)";')
+    gene_name_pattern = re.compile(r'gene_name "(\S+)"')
+    id_name = {}
+    c = Counter()
+    with open(gtf_file) as f:
+        for line in f:
+            if not line.strip():
+                continue
+            if line.startswith('#'):
+                continue
+            tabs = line.split('\t')
+            gtf_type, attributes = tabs[2], tabs[-1]
+            if gtf_type == 'gene':
+                gene_id = gene_id_pattern.findall(attributes)[-1]
+                gene_name = gene_name_pattern.findall(attributes)[-1]
+                c[gene_name] += 1
+                if c[gene_name] > 1:
+                    gene_name = f'{gene_name}_{c[gene_name]}'
+                id_name[gene_id] = gene_name
+    return id_name
+
+
+@log
 def process_read(read2_file, pattern_dict, barcode_dict, linker_dict):
     # if valid, return (True)
     metrics = defaultdict(int)
@@ -53,7 +203,7 @@ def process_read(read2_file, pattern_dict, barcode_dict, linker_dict):
             if hamming_correct(linker_dict[linker_name], seq_linker):
                 valid_linker = True
                 continue
-        
+
         if not valid_linker:
             metrics['Reads Unmapped Invalid Linker'] += 1
             continue
@@ -73,8 +223,8 @@ def process_read(read2_file, pattern_dict, barcode_dict, linker_dict):
         # mapped
         metrics['Reads Mapped'] += 1
         if metrics['Reads Mapped'] % 1000000 == 0:
-            logger1.info(str(metrics['Reads Mapped']) + " reads done.")
-        
+            process_read.logger.info(str(metrics['Reads Mapped']) + " reads done.")
+
     return res_dict, metrics
 
 
@@ -140,10 +290,18 @@ def hamming_distance(string1, string2):
 
 def gen_stat(df, stat_file):
     # 3cols: item count total_count
-    df['percent'] = df["count"] / df['total_count']
-    df['value'] = df.apply(
-        lambda x: f"{x['count']}({round(x['percent'] * 100, 2)}%)",
-        axis=1)
+
+    def add_percent(row):
+        count = row['count']
+        percent = count / row['total_count']
+        value = f'{format_number(count)}({round(percent * 100, 2)}%)'
+        return value
+    df.loc[~df['total_count'].isna(), 'value'] = df.loc[~df['total_count'].isna(), :].apply(
+        lambda row: add_percent(row), axis=1
+    )
+    df.loc[df['total_count'].isna(), 'value'] = df.loc[df['total_count'].isna(), :].apply(
+        lambda row: f'{format_number(row["count"])}', axis=1
+    )
     df = df.loc[:, ["item", "value"]]
     df.to_csv(stat_file, sep=":", header=None, index=False)
 
@@ -152,10 +310,14 @@ def get_fq(library_id, library_path):
     try:
         pattern1_1 = library_path + '/' + library_id + '*' + '_1.fq.gz'
         pattern1_2 = f'{library_path}/*{library_id}*R1.fastq.gz'
+        pattern1_3 = f'{library_path}/*{library_id}*R1_001.fastq.gz'
         pattern2_1 = library_path + '/' + library_id + '*' + '_2.fq.gz'
         pattern2_2 = f'{library_path}/*{library_id}*R2.fastq.gz'
-        fq1 = ",".join(glob.glob(pattern1_1) + glob.glob(pattern1_2))
-        fq2 = ",".join(glob.glob(pattern2_1) + glob.glob(pattern2_2))
+        pattern2_3 = f'{library_path}/*{library_id}*R2_001.fastq.gz'
+        fq1 = ",".join(glob.glob(pattern1_1) + glob.glob(pattern1_2) + glob.glob(pattern1_3))
+        fq2 = ",".join(glob.glob(pattern2_1) + glob.glob(pattern2_2) + glob.glob(pattern2_3))
+        if len(fq1) == 0:
+            sys.exit('Invalid fastq name pattern!')
     except IndexError as e:
         sys.exit("Mapfile Error:" + str(e))
     return fq1, fq2
@@ -207,8 +369,9 @@ job_end
     return res_cmd
 
 
-def merge_report(fq_dict, steps, last_step, sjm_cmd,
-                 sjm_order, logdir, conda, outdir, rm_files):
+def merge_report(
+    fq_dict, steps, last_step, sjm_cmd,
+    sjm_order, logdir, conda, outdir, rm_files):    
     step = "merge_report"
     steps_str = ",".join(steps)
     samples = ','.join(fq_dict.keys())
@@ -231,7 +394,8 @@ def format_number(number: int) -> str:
     return format(number, ",")
 
 
-def glob_genomeDir(genomeDir, logger1):
+@log
+def glob_genomeDir(genomeDir):
     refFlat = glob.glob(genomeDir + "/*.refFlat")
     if (len(refFlat) > 1):
         sys.exit("ERROR: Multiple refFlat file in " + genomeDir)
@@ -239,21 +403,23 @@ def glob_genomeDir(genomeDir, logger1):
         sys.exit("ERROR: refFlat file not found in " + genomeDir)
     else:
         refFlat = refFlat[0]
-        logger1.info("refFlat file found: " + refFlat)
+        glob_genomeDir.logger.info("refFlat file found: " + refFlat)
 
     gtf = glob.glob(genomeDir + "/*.gtf")
     if (len(gtf) == 0):
         sys.exit("ERROR: gtf file not found in " + genomeDir)
     elif (len(gtf) > 1):
         gtf = glob.glob(genomeDir + "/*.chr.gtf")
+        if (len(gtf) == 0):
+            sys.exit("ERROR: No chr gtf file in "+ genomeDir)
         if (len(gtf) > 1):
             sys.exit("ERROR: Multiple gtf file in " + genomeDir)
         else:
             gtf = gtf[0]
-            logger1.info("chr gtf file found: " + gtf)
+            glob_genomeDir.logger.info("chr gtf file found: " + gtf)
     else:
         gtf = gtf[0]
-        logger1.info("gtf file found: " + gtf)
+        glob_genomeDir.logger.info("gtf file found: " + gtf)
 
     return refFlat, gtf
 
@@ -374,6 +540,7 @@ def genDict(dim=3):
         return defaultdict(lambda: genDict(dim - 1))
 
 
+@log
 def downsample(bam, barcodes, percent):
     """
     calculate median_geneNum and saturation based on a given percent of reads
